@@ -100,7 +100,7 @@ linestruct *copy_node(const linestruct *src)
 {
 	linestruct *dst = nmalloc(sizeof(linestruct));
 
-	dst->data = mallocstrcpy(NULL, src->data);
+	dst->data = copy_of(src->data);
 	dst->next = src->next;
 	dst->prev = src->prev;
 	dst->lineno = src->lineno;
@@ -127,33 +127,33 @@ void splice_node(linestruct *afterthis, linestruct *newnode)
 }
 
 /* Disconnect a node from a linked list of linestructs and delete it. */
-void unlink_node(linestruct *fileptr)
+void unlink_node(linestruct *line)
 {
-	if (fileptr->prev != NULL)
-		fileptr->prev->next = fileptr->next;
-	if (fileptr->next != NULL)
-		fileptr->next->prev = fileptr->prev;
+	if (line->prev != NULL)
+		line->prev->next = line->next;
+	if (line->next != NULL)
+		line->next->prev = line->prev;
 
 	/* Update filebot when removing a node at the end of file. */
-	if (openfile && openfile->filebot == fileptr)
-		openfile->filebot = fileptr->prev;
+	if (openfile && openfile->filebot == line)
+		openfile->filebot = line->prev;
 
-	delete_node(fileptr);
+	delete_node(line);
 }
 
 /* Free the data structures in the given node. */
-void delete_node(linestruct *fileptr)
+void delete_node(linestruct *line)
 {
 #ifdef ENABLE_WRAPPING
 	/* If the spill-over line for hard-wrapping is deleted... */
-	if (fileptr == openfile->spillage_line)
+	if (line == openfile->spillage_line)
 		openfile->spillage_line = NULL;
 #endif
-	free(fileptr->data);
+	free(line->data);
 #ifdef ENABLE_COLOR
-	free(fileptr->multidata);
+	free(line->multidata);
 #endif
-	free(fileptr);
+	free(line);
 }
 
 /* Duplicate an entire linked list of linestructs. */
@@ -226,14 +226,14 @@ void partition_buffer(linestruct *top, size_t top_x,
 	 * top of the partition from it, and save the text before top_x. */
 	foreline = top->prev;
 	top->prev = NULL;
-	antedata = mallocstrncpy(NULL, top->data, top_x + 1);
+	antedata = measured_copy(top->data, top_x + 1);
 	antedata[top_x] = '\0';
 
 	/* Remember which line is below the bottom of the partition, detach the
 	 * bottom of the partition from it, and save the text after bot_x. */
 	hindline = bot->next;
 	bot->next = NULL;
-	postdata = mallocstrcpy(NULL, bot->data + bot_x);
+	postdata = copy_of(bot->data + bot_x);
 
 	/* At the end of the partition, remove all text after bot_x. */
 	bot->data[bot_x] = '\0';
@@ -342,7 +342,7 @@ void extract(linestruct *top, size_t top_x, linestruct *bot, size_t bot_x)
 
 	/* Since the text has now been saved, remove it from the file buffer. */
 	openfile->filetop = make_new_node(NULL);
-	openfile->filetop->data = mallocstrcpy(NULL, "");
+	openfile->filetop->data = copy_of("");
 	openfile->filebot = openfile->filetop;
 
 	/* Restore the current line and cursor position.  If the mark begins
@@ -463,11 +463,15 @@ void print_view_warning(void)
 	statusbar(_("Key is invalid in view mode"));
 }
 
-/* Indicate that something is disabled in restricted mode. */
-void show_restricted_warning(void)
+/* When in restricted mode, show a warning and return TRUE. */
+bool in_restricted_mode(void)
 {
-	statusbar(_("This function is disabled in restricted mode"));
-	beep();
+	if (ISSET(RESTRICTED)) {
+		statusbar(_("This function is disabled in restricted mode"));
+		beep();
+		return TRUE;
+	} else
+		return FALSE;
 }
 
 #ifndef ENABLE_HELP
@@ -500,7 +504,7 @@ void finish(void)
 	/* Restore the old terminal settings. */
 	tcsetattr(0, TCSANOW, &original_state);
 
-#ifdef ENABLE_NANORC
+#if defined(ENABLE_NANORC) || defined(ENABLE_HISTORIES)
 	display_rcfile_errors();
 #endif
 
@@ -1340,10 +1344,8 @@ void do_toggle(int flag)
 {
 	bool enabled;
 
-	if (flag == SUSPEND && ISSET(RESTRICTED)) {
-		show_restricted_warning();
+	if (flag == SUSPEND && in_restricted_mode())
 		return;
-	}
 
 	TOGGLE(flag);
 	focusing = FALSE;
@@ -1811,7 +1813,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 			output[i] = '\n';
 
 		/* Get the next multibyte character. */
-		charlen = parse_mbchar(output + i, onechar, NULL);
+		charlen = collect_char(output + i, onechar);
 
 		i += charlen;
 
@@ -1837,7 +1839,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 		if (openfile->last_action != ADD ||
 				openfile->current_undo->mark_begin_lineno != openfile->current->lineno ||
 				openfile->current_undo->mark_begin_x != openfile->current_x)
-			add_undo(ADD);
+			add_undo(ADD, NULL);
 
 		/* Note that current_x has not yet been incremented. */
 		if (openfile->current == openfile->mark &&
@@ -2442,11 +2444,11 @@ int main(int argc, char **argv)
 #ifdef ENABLE_JUSTIFY
 	/* Set the default value for things that weren't specified. */
 	if (punct == NULL)
-		punct = mallocstrcpy(NULL, "!.?");
+		punct = copy_of("!.?");
 	if (brackets == NULL)
-		brackets = mallocstrcpy(NULL, "\"')>]}");
+		brackets = copy_of("\"')>]}");
 	if (quotestr == NULL)
-		quotestr = mallocstrcpy(NULL, "^([ \t]*([!#%:;>|}]|/{2}))+");
+		quotestr = copy_of("^([ \t]*([!#%:;>|}]|/{2}))+");
 
 	/* Compile the quoting regex, and exit when it's invalid. */
 	quoterc = regcomp(&quotereg, quotestr, NANO_REG_EXTENDED);
@@ -2470,14 +2472,14 @@ int main(int argc, char **argv)
 		const char *spellenv = getenv("SPELL");
 
 		if (spellenv != NULL)
-			alt_speller = mallocstrcpy(NULL, spellenv);
+			alt_speller = copy_of(spellenv);
 	}
 #endif
 
 #ifndef NANO_TINY
 	/* If matchbrackets wasn't specified, set its default value. */
 	if (matchbrackets == NULL)
-		matchbrackets = mallocstrcpy(NULL, "(<[{)>]}");
+		matchbrackets = copy_of("(<[{)>]}");
 
 	/* If the whitespace option wasn't specified, set its default value. */
 	if (whitespace == NULL) {
@@ -2485,13 +2487,13 @@ int main(int argc, char **argv)
 		if (using_utf8()) {
 			/* A tab is shown as a Right-Pointing Double Angle Quotation Mark
 			 * (U+00BB), and a space as a Middle Dot (U+00B7). */
-			whitespace = mallocstrcpy(NULL, "\xC2\xBB\xC2\xB7");
+			whitespace = copy_of("\xC2\xBB\xC2\xB7");
 			whitelen[0] = 2;
 			whitelen[1] = 2;
 		} else
 #endif
 		{
-			whitespace = mallocstrcpy(NULL, ">.");
+			whitespace = copy_of(">.");
 			whitelen[0] = 1;
 			whitelen[1] = 1;
 		}
@@ -2499,7 +2501,7 @@ int main(int argc, char **argv)
 #endif /* !NANO_TINY */
 
 	/* Initialize the search string. */
-	last_search = mallocstrcpy(NULL, "");
+	last_search = copy_of("");
 	UNSET(BACKWARDS_SEARCH);
 
 	/* If tabsize wasn't specified, set its default value. */
@@ -2607,7 +2609,7 @@ int main(int argc, char **argv)
 
 			if (argv[optind][n] == '/' || argv[optind][n] == '?') {
 				if (argv[optind][n + 1]) {
-					searchstring = mallocstrcpy(NULL, &argv[optind][n + 1]);
+					searchstring = copy_of(&argv[optind][n + 1]);
 					if (argv[optind][n] == '?')
 						SET(BACKWARDS_SEARCH);
 				} else if (n == 1)
@@ -2683,8 +2685,8 @@ int main(int argc, char **argv)
 	prepare_for_display();
 
 #ifdef ENABLE_NANORC
-	if (rcfile_with_errors != NULL)
-		statusline(ALERT, _("Mistakes in '%s'"), rcfile_with_errors);
+	if (startup_problem != NULL)
+		statusline(ALERT, startup_problem);
 #endif
 
 #ifdef ENABLE_HELP
