@@ -18,9 +18,6 @@
  *                                                                        *
  **************************************************************************/
 
-#ifndef PROTO_H
-#define PROTO_H 1
-
 #include "nano.h"
 
 /* All external variables.  See global.c for their descriptions. */
@@ -55,6 +52,8 @@ extern bool more_than_one;
 
 extern bool also_the_last;
 
+extern bool is_shorter;
+
 extern int didfind;
 
 extern int controlleft, controlright;
@@ -75,7 +74,8 @@ extern int shiftaltup, shiftaltdown;
 #endif
 
 #ifdef ENABLED_WRAPORJUSTIFY
-extern ssize_t fill, wrap_at;
+extern ssize_t fill;
+extern size_t wrap_at;
 #endif
 
 extern char *last_search;
@@ -173,8 +173,8 @@ extern int interface_color_pair[NUMBER_OF_ELEMENTS];
 
 extern char *homedir;
 extern char *statedir;
-#ifdef ENABLE_NANORC
-extern char *rcfile_with_errors;
+#if defined(ENABLE_NANORC) || defined(ENABLE_HISTORIES)
+extern char *startup_problem;
 #endif
 
 extern bool spotlighted;
@@ -210,10 +210,14 @@ bool is_ascii_cntrl_char(int c);
 bool is_cntrl_mbchar(const char *c);
 bool is_word_mbchar(const char *c, bool allow_punct);
 char control_mbrep(const char *c, bool isdata);
+#ifdef ENABLE_UTF8
 int mbwidth(const char *c);
+#endif
 char *make_mbchar(long chr, int *chr_mb_len);
 int char_length(const char *pointer);
-int parse_mbchar(const char *buf, char *chr, size_t *col);
+size_t mbstrlen(const char *s);
+int collect_char(const char *buf, char *chr);
+int advance_over(const char *string, size_t *column);
 size_t step_left(const char *buf, size_t pos);
 size_t step_right(const char *buf, size_t pos);
 int mbstrcasecmp(const char *s1, const char *s2);
@@ -221,7 +225,6 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n);
 char *mbstrcasestr(const char *haystack, const char *needle);
 char *revstrstr(const char *haystack, const char *needle, const char *index);
 char *mbrevstrcasestr(const char *haystack, const char *needle, const char *index);
-size_t mbstrlen(const char *s);
 #if !defined(NANO_TINY) || defined(ENABLE_JUSTIFY)
 char *mbstrchr(const char *s, const char *c);
 #endif
@@ -233,6 +236,7 @@ char *mbrevstrpbrk(const char *head, const char *accept, const char *index);
 #if defined(ENABLE_NANORC) && (!defined(NANO_TINY) || defined(ENABLE_JUSTIFY))
 bool has_blank_char(const char *s);
 #endif
+bool white_string(const char *s);
 #ifdef ENABLE_UTF8
 bool is_valid_unicode(wchar_t wc);
 #endif
@@ -242,8 +246,8 @@ bool is_valid_unicode(wchar_t wc);
 void set_colorpairs(void);
 void color_init(void);
 void color_update(void);
+void set_up_multicache(linestruct *line);
 void check_the_multis(linestruct *line);
-void alloc_multidata_if_needed(linestruct *fileptr);
 void precalc_multicolorinfo(void);
 #endif
 
@@ -255,7 +259,7 @@ void chop_previous_word(void);
 void chop_next_word(void);
 void cut_marked(bool *right_side_up);
 #endif
-void do_cut_text(bool copying, bool marked, bool until_eof, bool append);
+void do_snip(bool copying, bool marked, bool until_eof, bool append);
 bool is_cuttable(bool test_cliff);
 void cut_text(void);
 #ifndef NANO_TINY
@@ -270,7 +274,8 @@ void make_new_buffer(void);
 void set_modified(void);
 bool open_buffer(const char *filename, bool new_buffer);
 #ifdef ENABLE_SPELLER
-bool replace_buffer(const char *filename, undo_type action, bool marked);
+bool replace_buffer(const char *filename, undo_type action, bool marked,
+		const char *operation);
 #endif
 void prepare_for_display(void);
 #ifdef ENABLE_MULTIBUFFER
@@ -314,9 +319,10 @@ char *input_tab(char *buf, bool allow_files, size_t *place,
 #endif
 
 /* Some functions in global.c. */
-size_t length_of_list(int menu);
 const keystruct *first_sc_for(int menu, void (*func)(void));
 int the_code_for(void (*func)(void), int defaultval);
+size_t shown_entries_for(int menu);
+const keystruct *get_shortcut(int *kbinput);
 functionptrtype func_from_key(int *kbinput);
 int keycode_from_string(const char *keystring);
 void assign_keyinfo(keystruct *s, const char *keystring, const int keycode);
@@ -330,7 +336,7 @@ char *menu_to_name(int menu);
 
 /* All functions in help.c. */
 #ifdef ENABLE_HELP
-void wrap_help_text_into_buffer();
+void wrap_help_text_into_buffer(void);
 void do_help(void);
 void help_init(void);
 functionptrtype parse_help_input(int *kbinput);
@@ -396,12 +402,12 @@ void free_lines(linestruct *src);
 void renumber_from(linestruct *line);
 void partition_buffer(linestruct *top, size_t top_x,
 		linestruct *bot, size_t bot_x);
-void unpartition_buffer();
+void unpartition_buffer(void);
 void extract(linestruct *top, size_t top_x, linestruct *bot, size_t bot_x);
 void ingraft_buffer(linestruct *somebuffer);
 void copy_from_buffer(linestruct *somebuffer);
 void print_view_warning(void);
-void show_restricted_warning(void);
+bool in_restricted_mode(void);
 #ifndef ENABLE_HELP
 void say_there_is_no_help(void);
 #endif
@@ -464,8 +470,11 @@ int do_prompt(bool allow_tabs, bool allow_files,
 int do_yesno_prompt(bool all, const char *msg);
 
 /* Most functions in rcfile.c. */
+#if defined(ENABLE_NANORC) || defined(ENABLE_HISTORIES)
+void display_rcfile_errors(void);
+void jot_error(const char *msg, ...);
+#endif
 #ifdef ENABLE_NANORC
-void display_rcfile_errors();
 #ifdef ENABLE_COLOR
 void parse_one_include(char *file, syntaxtype *syntax);
 void grab_and_store(const char *kind, char *ptr, regexlisttype **storage);
@@ -476,6 +485,8 @@ void do_rcfiles(void);
 #endif /* ENABLE_NANORC */
 
 /* Most functions in search.c. */
+bool regexp_init(const char *regexp);
+void tidy_up_after_search(void);
 int findnextstr(const char *needle, bool whole_word_only, int modus,
 		size_t *match_len, bool skipone, const linestruct *begin, size_t begin_x);
 void do_search(void);
@@ -506,7 +517,6 @@ void do_tab(void);
 void do_indent(void);
 void do_unindent(void);
 #endif
-bool white_string(const char *s);
 #ifdef ENABLE_COMMENT
 void do_comment(void);
 #endif
@@ -516,8 +526,8 @@ void do_enter(void);
 #ifndef NANO_TINY
 RETSIGTYPE cancel_command(int signal);
 bool execute_command(const char *command);
-void discard_until(const undo *thisitem, openfilestruct *thefile, bool keep);
-void add_undo(undo_type action);
+void discard_until(const undostruct *thisitem, openfilestruct *thefile, bool keep);
+void add_undo(undo_type action, const char *message);
 void update_multiline_undo(ssize_t lineno, char *indentation);
 void update_undo(undo_type action);
 #endif /* !NANO_TINY */
@@ -531,6 +541,7 @@ ssize_t break_line(const char *line, ssize_t goal, bool snap_at_nl);
 size_t indent_length(const char *line);
 #endif
 #ifdef ENABLE_JUSTIFY
+size_t quote_length(const char *line);
 bool begpar(const linestruct *const foo, int depth);
 bool inpar(const linestruct *const foo);
 void do_justify(bool full_justify);
@@ -542,6 +553,7 @@ void do_spell(void);
 #endif
 #ifdef ENABLE_COLOR
 void do_linter(void);
+void do_formatter(void);
 #endif
 #ifndef NANO_TINY
 void do_wordlinechar_count(void);
@@ -558,8 +570,8 @@ int digits(ssize_t n);
 #endif
 bool parse_num(const char *str, ssize_t *val);
 bool parse_line_column(const char *str, ssize_t *line, ssize_t *column);
-void unsunder(char *str, size_t true_len);
-void sunder(char *str);
+void unsunder(char *string, size_t length);
+void sunder(char *string);
 #if !defined(ENABLE_TINY) || defined(ENABLE_TABCOMP) || defined(ENABLE_BROWSER)
 void free_chararray(char **array, size_t len);
 #endif
@@ -570,8 +582,9 @@ const char *strstrwrapper(const char *haystack, const char *needle,
 		const char *start);
 void *nmalloc(size_t howmuch);
 void *nrealloc(void *ptr, size_t howmuch);
-char *mallocstrncpy(char *dest, const char *src, size_t n);
+char *measured_copy(const char *src, size_t n);
 char *mallocstrcpy(char *dest, const char *src);
+char *copy_of(const char *string);
 char *free_and_assign(char *dest, char *src);
 size_t get_page_start(size_t column);
 size_t xplustabs(void);
@@ -612,8 +625,6 @@ int *parse_verbatim_kbinput(WINDOW *win, size_t *count);
 #ifdef ENABLE_MOUSE
 int get_mouseinput(int *mouse_row, int *mouse_col, bool allow_shortcuts);
 #endif
-const keystruct *get_shortcut(int *kbinput);
-void blank_row(WINDOW *win, int y, int x, int n);
 void blank_edit(void);
 void blank_statusbar(void);
 void wipe_statusbar(void);
@@ -622,14 +633,12 @@ void check_statusblank(void);
 char *display_string(const char *buf, size_t column, size_t span,
 						bool isdata, bool isprompt);
 void titlebar(const char *path);
+void statusline(message_type importance, const char *msg, ...);
 void statusbar(const char *msg);
 void warn_and_shortly_pause(const char *msg);
-void statusline(message_type importance, const char *msg, ...);
 void bottombars(int menu);
 void post_one_key(const char *keystroke, const char *tag, int width);
 void place_the_cursor(void);
-void edit_draw(linestruct *fileptr, const char *converted,
-		int line, size_t from_col);
 int update_line(linestruct *fileptr, size_t index);
 #ifndef NANO_TINY
 int update_softwrapped_line(linestruct *fileptr);
@@ -693,5 +702,3 @@ void flip_newbuffer(void);
 #endif
 void discard_buffer(void);
 void do_cancel(void);
-
-#endif /* !PROTO_H */
